from dataclasses import dataclass, field, replace, fields as dataclass_fields, Field
from typing import ClassVar, Type, Any
import sys
import dataclasses

class AutoInitFalseMeta(type):
    def __new__(mcs, name, bases, namespace):
        if name != 'BaseIDs' and any(issubclass(base, BaseIDs) for base in bases):
            annotations = namespace.get('__annotations__', {})
            for fname, ftype in annotations.items():
                if not fname.startswith('_'):
                    orig_field = namespace.get(fname, None)
                    if isinstance(orig_field, Field):
                        namespace[fname] = replace(orig_field, init=False)
                    else:
                        namespace[fname] = field(default=orig_field, init=False)
        return super().__new__(mcs, name, bases, namespace)


@dataclass(frozen=True)
class BaseIDs(metaclass=AutoInitFalseMeta):
    _prefix: ClassVar[str]

    def __post_init__(self):
        prefixes = []
        for cls in reversed(self.__class__.__mro__):
            if hasattr(cls, '_prefix'):
                prefixes.append(getattr(cls, '_prefix'))

        concatenated_prefix = '-'.join(prefixes)

        for f in dataclass_fields(self):
            if not f.name.startswith("_"):
                if f.default is not None:
                    object.__setattr__(self, f.name, f"{concatenated_prefix}-{f.default}")
                else:
                    formatted_field_name = f.name.replace('_', '-')
                    object.__setattr__(self, f.name, f"{concatenated_prefix}-{formatted_field_name}")
                print(f"{f.name} => {getattr(self, f.name)}")


# Author: Dencyuman <takotakocyuman@gmail.com>
# Copyright (c) 2024 Dencyuman
# License: MIT License

from setuptools import setup
import dash_id_manager

DESCRIPTION = "Efficient id manager for Plotly Dash."
LONG_DESCRIPTION = "Efficient id manager for Plotly Dash."
NAME = 'dash-id-manager'
AUTHOR = 'Dencyuman'
AUTHOR_EMAIL = 'takotakocyuman@gmail.com'
URL = 'https://github.com/Dencyuman/dash-id-manager'
LICENSE = 'MIT License'
DOWNLOAD_URL = 'https://github.com/Dencyuman/dash-id-manager'
VERSION = dash_id_manager.__version__
PYTHON_REQUIRES = ">=3.7"

INSTALL_REQUIRES = []

EXTRAS_REQUIRE = {}

PACKAGES = [
    'dash_id_manager',
]

CLASSIFIERS = []

setup(name=NAME,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      maintainer=AUTHOR,
      maintainer_email=AUTHOR_EMAIL,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      license=LICENSE,
      url=URL,
      version=VERSION,
      download_url=DOWNLOAD_URL,
      python_requires=PYTHON_REQUIRES,
      install_requires=INSTALL_REQUIRES,
      extras_require=EXTRAS_REQUIRE,
      packages=PACKAGES,
      classifiers=CLASSIFIERS
)
